// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QJsonValue>
#include <QTimer>
#include <QWebSocket>

namespace PackageFromDesignStudio {
using namespace Qt::Literals;
constexpr auto designStudioReady = "designStudioReady"_L1;
constexpr auto projectData = "projectData"_L1;
constexpr auto stopRunningProject = "stopRunningProject"_L1;
}; // namespace PackageFromDesignStudio

namespace PackageToDesignStudio {
using namespace Qt::Literals;
constexpr auto deviceInfo = "deviceInfo"_L1;
constexpr auto projectRunning = "projectRunning"_L1;
constexpr auto projectStopped = "projectStopped"_L1;
constexpr auto projectLogs = "projectLogs"_L1;
}; // namespace PackageToDesignStudio

class DesignStudio : public QObject
{
    Q_OBJECT
public:
    DesignStudio(QWebSocket *socket, const QString &m_deviceUuid, QObject *parent = nullptr);

    // Getters
    QString ipv4Addr() const;
    QString id() const;

    // Send data
    void sendDeviceInfo();
    void sendProjectRunning();
    void sendProjectStopped();
    void sendProjectLogs(const QString &logs);

private:
    // Network
    QScopedPointer<QWebSocket> m_socket;
    bool m_socketWasConnected;

    QTimer m_pingTimer;
    QTimer m_pongTimer;

    // DS data
    QString m_id;
    QString m_deviceUuid;

    // DS comm
    void sendData(const QLatin1String &dataType, const QJsonValue &data = QJsonValue());

    // Internal
    void initPingPong();
    void initSocket();

private slots:
    void processTextMessage(const QString &data);
    void processBinaryMessage(const QByteArray &data);

signals:
    // socket signals
    void disconnected(const QString &id);
    void connected(const QString &id);

    // DS signals
    void idReceived(const QString &id, const QString &ipv4Addr);
    void projectIncoming();
    void projectReceived(const QString &id, const QByteArray &data);
    void projectStopRequested(const QString &id);
};
